@@ -20,6 +20,7 @@ import pickle
 from collections import Mapping
 
 import pandas as pd
+from pandas import np
 import dateutil
 import timestring
 from sklearn.preprocessing import MinMaxScaler
@@ -34,7 +35,7 @@ from pugnlp.util import dict2obj
 from nlpia.data.loaders import read_csv
 
 from .constants import logging, SECRETS
-from .constants import DEFAULT_JSON_DIR, DEFAULT_JSON_PATH, DEFAULT_DB_CSV_PATH, DEFAULT_META_PATH, DEFAULT_MODEL_PATH
+from .constants import DEFAULT_JSON_DIR, DEFAULT_JSON_PATH, DEFAULT_DB_CSV_PATH, DEFAULT_META_PATH, DEFAULT_MODEL_PATH, DEFAULT_HUMAN_PATH
 from .constants import NAME_STRIP_CHRS, CFG
 
 DATADOG_OPTIONS = SECRETS.datadog.__dict__
@@ -141,7 +142,7 @@ def read_series(file_or_path=None, i=0):
     return ts
 
 
-def normalize_metric_name(display_name, scope='*'):
+def normalize_metric_name(display_name, scope='*', max_len=80):
     """Combine host FQDN and metric display_name to create unique "key" for each metric"""
     if '{' in display_name or ':' in display_name:
         name = display_name.strip().lower()
@@ -155,7 +156,7 @@ def normalize_metric_name(display_name, scope='*'):
         name = ':'.join([s for s in [hostname, metricname] if len(s)])
 
     name = re.sub(r'\s', '', name)
-    name = name[:80]
+    name = name[:max_len]
     return name
 
 
@@ -270,13 +271,21 @@ def is_anomalous(df, thresholds=None):
     if thresholds is None:
         thresholds = [(q['query'], q['threshold']) for q in CFG.queries if q['query'] in df.columns]
     queries, values = zip(*thresholds)
+    lows, highs = zip(*values)
 
     ans = pd.DataFrame(pd.np.zeros((len(df), len(queries) + 1)).astype(bool),
-                       columns=['anomaly__' + k for k in queries] + ['anomaly__any'],
+                       columns=['anomaly__<{}&>{}__{}'.format(low, high, query) for query, low, high in zip(queries, lows, highs)] + ['anomaly__any'],
                        index=df.index)
-    for dfk, ansk, value in zip(queries, ans.columns, values):
+    for dfk, ansk, (low, high) in zip(queries, ans.columns, values):
+        high = float(high) if isinstance(high, str) else high
+        low = float(low) if isinstance(low, str) else low
+        high = np.nan if pd.isnull(high) else high
+        low = np.nan if pd.isnull(low) else low
         if dfk in queries:
-            ans[ansk] = df[dfk] > value
+            if not pd.isnull(high):
+                ans[ansk] |= df[dfk] > high
+            if not pd.isnull(low):
+                ans[ansk] |= df[dfk] < low
             ans['anomaly__any'] |= ans[ansk]
         else:
             logger.error('No threshold defined for {}'.format(dfk))
@@ -286,14 +295,58 @@ def is_anomalous(df, thresholds=None):
 def join_spans(spans):
     spans = list(spans)
     joined_spans = [list(spans[0])]
-    for i, (start, stop) in enumerate(spans[1:]):
-        if start > joined_spans[i][1]:
-            joined_spans += [[start, stop]]
-        else:
-            joined_spans[i - 1][1] = stop
+    if len(spans) > 1:
+        for i, (start, stop) in enumerate(spans[1:]):
+            if start > joined_spans[i][1]:
+                joined_spans += [[start, stop]]
+            else:
+                # if the span is zero length or started before it stopped then don't add it, just shorten the previous span
+                joined_spans[i - 1][1] = stop
     return joined_spans
 
 
+def ask_if_anomalous(new_spans, human_labels_path=DEFAULT_HUMAN_PATH):
+    """Console input by user to confirm or deny anomalous timespans"""
+    try:
+        df = read_csv(human_labels_path)
+    except IOError:
+        df = pd.DataFrame(columns='start end human_label'.split())
+
+    columns = df.columns
+    print('Anomalous Time Spans Detected in Past 24 hours:')
+    print(pd.DataFrame(new_spans, columns=columns[:2]))
+    print('Refer to the time-series.html plot for this time period to determine whether these time spans were truly anomalous.')
+    human_labels = pd.np.zeros(len(new_spans), dtype=int)
+    for i, (start, end) in enumerate(new_spans):
+        print("{}: {} to {}".format(i, start, end))
+        ans = input("Is the time span above anomalous [Y]/N/Yall/Nall? ") or 'Y'
+        ans = ans.strip()
+        print("GOT: {}".format(ans))
+        if re.match(r'^(y|Y|Yes|YES|yes|yep|yup)$', ans):
+            human_labels[i] = 1
+            print("YES it was anomalous")
+        elif ans.lower().endswith('all'):
+            if ans.lower() == 'yall':
+                print("YES, ALL were anomalous!!")
+                human_labels[i:] = np.array([1] * len(human_labels[i:]))
+                i = len(new_spans)
+                break
+            elif ans.lower() == 'nall':
+                print("NO, ALL were clean...")
+                i = len(new_spans)
+                break
+        else:
+            print("NO it was clean")
+
+    dfnew = pd.DataFrame(new_spans, columns=df.columns[:2])
+    dfnew[df.columns[-1]] = human_labels
+    df = df.append(dfnew, ignore_index=True)
+    print(df)
+    print(dfnew)
+    df.to_csv(human_labels_path)
+    return df
+
+
 def plot_all(df=None, fillna_method='ffill', dropna=False, filename='time-series.html'):
     df = DEFAULT_JSON_DIR if df is None else df
     df = clean_dd_all(df, fillna_method=fillna_method, dropna=dropna).astype(float) if isinstance(df, str) else pd.DataFrame(df)
@@ -304,7 +357,7 @@ def plot_all(df=None, fillna_method='ffill', dropna=False, filename='time-series
     stops = list(df.index[anom_spans < 0])
     if len(stops) == len(starts) - 1:
         stops += [df.index.values[-1]]
-    anom_spans = join_spans(join_spans(zip(starts, stops)))
+    anom_spans = join_spans(zip(starts, stops))
 
     print(anom_spans)
 
@@ -324,15 +377,21 @@ def plot_predictions(df=None, fillna_method='ffill', dropna=False, filename='tim
     """
     df = DEFAULT_JSON_DIR if df is None else df
     df = clean_dd_all(df, fillna_method=fillna_method, dropna=dropna).astype(float) if isinstance(df, str) else pd.DataFrame(df)
-
+    columns = df.columns
     rf = pickle.load(open(DEFAULT_MODEL_PATH, 'rb'))
+
+    logger.info(rf.columns)
+    columns = getattr(rf, 'columns', getattr(rf, 'feature_names_'))
+    if columns is None:
+        logger.warn("Unable to find a columns attribute on the model, so just assuming that they are the same ones in the latest database columns")
+        columns = df.columns[:rf.n_features_]
+
     # df = clean_dd_all(df)
-    thresholds = [(q['query'], q['threshold']) for q in CFG.queries if q['query'] in df.columns]
-    predictions = rf.predict(df.values)
+    # thresholds = [(q['query'], q['threshold']) for q in CFG.queries if q['query'] in df.columns]
+    predictions = rf.predict(df[columns].values)
     anoms = pd.DataFrame(predictions,
                          columns=[q['query'] for q in CFG.queries if q['query'] in df.columns] + ['anomaly__any'],
                          index=df.index.values)
-    # FIXME: this should be an ordereddict or just a list of tuples
     # anoms = is_anomalous(df)  # manually-determined thresholds on queries from Chase
     anoms['anomaly__any'].iloc[0] = 0
     anoms['anomaly__any'].iloc[-1] = 0
@@ -347,16 +406,17 @@ def plot_predictions(df=None, fillna_method='ffill', dropna=False, filename='tim
     else:
         anom_spans = []
 
-    print('Anomalous Time Spans:')
-    print(anom_spans)
-
     df = df[[c for c in anoms.columns if c in df.columns]]
     df['Num. Anomalous Monitors'] = (anoms[anoms.columns[:-1]].sum(axis=1) + .01)
+    df.columns = [normalize_metric_name(c, max_len=64) for c in df.columns]
     offline.plot(df.iplot(
         asFigure=True, xTitle='Date-Time', yTitle='Monitor Value', kind='scatter', logy=True,
         vspan=anom_spans),
         filename=filename,
         )
+
+    ask_if_anomalous(anom_spans)
+
     return df
 
 
@@ -592,19 +652,20 @@ def update_meta(meta=None, drop=False):
     """Query DataDog to retrieve all the host and metric names and save them to meta.json"""
     empty_meta = {'hosts': [], 'metrics': [], 'monitors': {}}
     metapath = meta
+    meta = empty_meta
     if not isinstance(metapath, str):
         metapath = DEFAULT_META_PATH
-        if drop:
-            meta = empty_meta
-    try:
-        with open(metapath, 'rt') as f:
-            meta = json.load(f)
-    except IOError:
-        meta = empty_meta
+    if not drop:
+        try:
+            with open(metapath, 'rt') as f:
+                meta.update(json.load(f))
+        except IOError:
+            pass
     new_meta = get_meta()
     monitors = dd.api.monitors.Monitor.get_all(group_states=['all'])
     monitor_names = [monitor_dict['name'] for monitor_dict in monitors]
-    new_meta['monitors'] = dict(zip(monitor_names, monitors))
+    monitor_queries = [monitor_dict['query'] for monitor_dict in monitors]
+    new_meta['monitors'] = dict(zip(monitor_names, monitor_queries))
     for k in ['hosts', 'metrics']:
         meta[k] = sorted(set(meta[k]).union(set(new_meta[k])))
     for k in ['monitors']:
